# Linear Normal model to analyse abundance indices of Thiof fish in Senegal
# Chapter  6

# This program DOES NOT run any BUGS program

# It uses standard R functions and
# - compute the closed form posterior probability of parameters
# (using conjugate propoerties of linear models)
# - compute the odd-ratio for two competing models

# -------------------------------------------------------------------------


rm(list = ls())

library(lattice)
library(stats4)
library(MASS)

dat <- read.table(file = "data_lm_AbundanceIndicesThiof_table.txt", header = TRUE, sep = "")


# Compute catch per unit effort and join the new variable to the table
# ----------------------------------------------------

dat$CPUE <- dat$Catches/dat$Effort
head(dat)
names(dat)


# Declare variables as factors (except Year)
# ----------------------------------------------------

dat$Month <- as.factor(dat$Month)
dat$Season <- as.factor(dat$Season)
dat$Zone <- as.factor(dat$Zone)
dat$Arbor <- as.factor(dat$Arbor)



# System of constraints "effects sun to zero"
# ----------------------------------------------------

contrasts(dat$Month) <- contr.sum(levels(dat$Month))
contrasts(dat$Season) <- contr.sum(levels(dat$Season))
contrasts(dat$Zone) <- contr.sum(levels(dat$Zone))
contrasts(dat$Arbor) <- contr.sum(levels(dat$Arbor))



# ------------------------------------------------------------------------------
# Bayesian analysis
# ------------------------------------------------------------------------------


# BAYESIAN ANALYSIS - Model 1
# log(CPUE) = Constant(1) + Season(1) + Zone(3) + Year(1)
# ---------------------------------------------------------------------------

n.param <- 6

n <- length(dat$CPUE)
y <- matrix(log(dat$CPUE),nrow=n, nc=1)
X <- matrix(NA,nrow=n,nc=n.param)

X[,1]<-rep(1,n)

X[,2] <- 1*(dat$Season==1) - 1*(dat$Season==2)

for(i in 1:3)
{
X[,i+2] <- 1*(dat$Zone==levels(dat$Zone)[i]) - 1*(dat$Zone==levels(dat$Zone)[4])
}

X[,6] <- as.numeric(dat$Year)


c = n
betahat = solve(t(X)%*%X)%*%t(X)%*%y
betapost = c/(c+1)*betahat
Sy <- (t(y-X%*%betahat)%*%(y-X%*%betahat)+1/(n+1)*t(X%*%betahat)%*%(X%*%betahat))
means2post= (t(y-X%*%betahat)%*%(y-X%*%betahat)+1/(c+1)*t(X%*%betahat)%*%(X%*%betahat))/n
approxvarbetapost=solve(t(X)%*%X)*as.numeric(means2post)*c/(1+c)
sqrt(diag(approxvarbetapost))
round(cbind(betapost,sqrt(diag(approxvarbetapost)),betapost+2*sqrt(diag(approxvarbetapost)),betapost-2*sqrt(diag(approxvarbetapost))),3)

# Log-predictive
lnpredict1 <- 0.5*det((t(X)%*%X)/c,log=T)-0.5*det((t(X)%*%X)*(1+c)/c,log=T)-n/2*log(Sy)
lnpredict1

# Comparison of parameter estimate with Results of a Frequentist analysis
lm1 <- lm(log(CPUE) ~ Zone + Season + Year, data=dat)
summary(lm1)


# BAYESIAN ANALYSIS - Model 2
# log(CPUE) = Constant(1) + Season(1) + Zone(3 = 4-1) +  Season%in%Year(2)
# ---------------------------------------------------------------------------

n.param <- 7

n <- length(dat$CPUE)
y <- matrix(log(dat$CPUE),nrow=n, nc=1)
X <- matrix(NA,nrow=n,nc=n.param)

X[,1]<-rep(1,n)

X[,2] <- 1*(dat$Season==1) - 1*(dat$Season==2)

for(i in 1:3)
{
X[,i+2] <- 1*(dat$Zone==levels(dat$Zone)[i]) - 1*(dat$Zone==levels(dat$Zone)[4])
}

for(i in 1:2)
{
X[,i+5] <- as.numeric(dat$Year)*(dat$Season==levels(dat$Season)[i])
}

c = n
betahat = solve(t(X)%*%X)%*%t(X)%*%y
betapost = c/(c+1)*betahat
Sy <- (t(y-X%*%betahat)%*%(y-X%*%betahat)+1/(n+1)*t(X%*%betahat)%*%(X%*%betahat))
means2post= (t(y-X%*%betahat)%*%(y-X%*%betahat)+1/(c+1)*t(X%*%betahat)%*%(X%*%betahat))/n
approxvarbetapost=solve(t(X)%*%X)*as.numeric(means2post)*c/(1+c)
sqrt(diag(approxvarbetapost))
round(cbind(betapost,sqrt(diag(approxvarbetapost)),betapost+2*sqrt(diag(approxvarbetapost)),betapost-2*sqrt(diag(approxvarbetapost))),3)

# Log-predictive
lnpredict2 <- 0.5*det((t(X)%*%X)/c,log=T)-0.5*det((t(X)%*%X)*(1+c)/c,log=T)-n/2*log(Sy)
lnpredict2

# Comparison of parameter estimate with Results of a Frequentist analysis
lm2 <- lm(log(CPUE) ~ Zone + Season + Season%in%Year, data=dat)
summary(lm2)
